#!/bin/bash

function mii {
    /usr/local/sbin/mii $*
}

function miiphy_speed {
	local reg

	reg=`mii R $1 0x0A`
	if [ $? != 0 ]; then
		echo PHY 1000BT Status read failed >&2
	else
		if (( $reg != 0xFFFF )); then
			if (( ($reg & 0x0c00) != 0 )); then
				echo _1000BASET
				return
			fi
		fi
	fi

	# Check Basic Management Control Register first.
	reg=`mii R $1 0x00`
	if [ $? != 0 ]; then
		echo PHY speed read failed, assuming 10bT >&2
		echo _10BASET
		return
	fi

	# Check if auto-negotiation is on.
	if (( $reg & 0x1000 )); then
		# Get auto-negotiation results.
		reg=`mii R $1 0x05`
		if [ $? != 0 ]; then
			echo PHY AN speed read failed, assuming 10bT >&2
			echo _10BASET
			return
		fi

		if (( $reg & 0x0380 )); then
			echo _100BASET
			return
		else
			echo _10BASET
			return
		fi
	# Get speed from basic control settings.
	elif (( $reg & 0x2000 )); then
		echo _100BASET
		return
	else
		echo _10BASET
		return
	fi
}

if (( (`mii R 0x10 0x03` & 0xfff0) == 0x3750 )); then
	echo "Marvell 88E6351 Switch"
	echo -n n3 > /dev/gpio	# enable ports signal
	echo -n p2 > /dev/gpio	# reset the switch signal
	exit 0
fi

echo -n f3 > /dev/gpio	# enable ports signal
echo -n p2 > /dev/gpio	# reset the switch signal

sleep 1

mii w 0x1b 0x04 0x0001

sleep 1

if (( `mii R 0x08 0x02` != 0x0141 )); then
	mii w 0x1b 0x04 0x4001
	exit 0
fi

if (( (`mii R 0x08 0x03` & 0xfff0) != 0x0dc0 )); then
	mii w 0x1b 0x04 0x4001
	exit 0
fi

echo "Marvell 88E1322 PHY"

mii w 0x08 0x16 0x00ff	# page 255
mii w 0x08 0x18 0x2800
mii w 0x08 0x17 0x2001

mii w 0x09 0x16 0x00ff	# page 255
mii w 0x09 0x18 0x2800
mii w 0x09 0x17 0x2001

mii w 0x0a 0x16 0x00ff	# page 255
mii w 0x0a 0x18 0x2800
mii w 0x0a 0x17 0x2001

mii w 0x0b 0x16 0x00ff	# page 255
mii w 0x0b 0x18 0x2800
mii w 0x0b 0x17 0x2001

mii w 0x08 0x16 0x0000	# page 0
mii w 0x08 0x1d 0x0003
mii w 0x08 0x1e 0x0002
mii w 0x08 0x1d 0x0000

mii w 0x09 0x16 0x0000	# page 0
mii w 0x09 0x1d 0x0003
mii w 0x09 0x1e 0x0002
mii w 0x09 0x1d 0x0000

mii w 0x0a 0x16 0x0000	# page 0
mii w 0x0a 0x1d 0x0003
mii w 0x0a 0x1e 0x0002
mii w 0x0a 0x1d 0x0000

mii w 0x0b 0x16 0x0000	# page 0
mii w 0x0b 0x1d 0x0003
mii w 0x0b 0x1e 0x0002
mii w 0x0b 0x1d 0x0000

mii w 0x08 0x16 0x0006	# page 6
mii w 0x08 0x14 0x8205

mii w 0x0a 0x16 0x0006	# page 6
mii w 0x0a 0x14 0x8205

mii w 0x08 0x16 0x0004	# page 4
mii w 0x08 0x00 0x9140
mii w 0x08 0x1b 0x7e03

mii w 0x09 0x16 0x0004	# page 4
mii w 0x09 0x00 0x9140

mii w 0x0a 0x16 0x0004	# page 4
mii w 0x0a 0x00 0x9140

mii w 0x0b 0x16 0x0004	# page 4
mii w 0x0b 0x00 0x9140

mii w 0x09 0x16 0x0000	# page 0
mii w 0x09 0x10 0x3360
mii w 0x09 0x00 0xb100

mii w 0x0b 0x16 0x0000	# page 0
mii w 0x0b 0x10 0x3360
mii w 0x0b 0x00 0xb100

# un-shut SGMII PHYs

mii w 0x08 0x16 0x0001  # page 1
mii w 0x08 0x00 0x1140

mii w 0x0a 0x16 0x0001  # page 1
mii w 0x0a 0x00 0x1140

isrh=`cat /proc/boardid`
if [  $isrh -le 3 ]; then
sleep 4

if [ `miiphy_speed 0x09` == _1000BASET ]; then
	mii w 0x18 0x01 0xc13e	# 1000BASE-T
else
	mii w 0x18 0x01 0xc13d	# 100BASE-T
fi

if [ `miiphy_speed 0x0b` == _1000BASET ]; then
	mii w 0x19 0x01 0xc53e	# 1000BASE-T
else
	mii w 0x19 0x01 0xc53d	# 100BASE-T
fi

mii w 0x1b 0x04 0x4001

fi