#!/bin/sh
#
#/etc/aidc/ps.sh
#
#input:
# $1 - "all" proccesses ; "app" only ours; specidfic: aidc/trapsender/....
# $2 - sleep time
# $3 - number of runs (0 - never ends)


PARAMETERS="pid,fname,%mem,rss,vsize,sz,%cpu,etime,stat,rtprio"
PROCESSES="aidc,trapsenderd,snmpd,subagentx,web_idc,nms_idc,lighttpd,HSL,imi,nsm,onmd,mstpd,rmond"
PRINT_SUM_INFO="free |grep -v Swap;sar 1 1 |grep -1 idle"
cnt=1

# ********************* HELP *****************************
if [ "$1" = ? ]; then
 echo "---- Proc Track help ---- 
 *** General:
 This script track the process memory allocation and CPU usage.
 The output is printed on the screen and on /etc/aidc/proc_track.log.
 This log will be added to the unit info tar file.

 *** Usage:
 proc_track.sh <Tracking Process> <Polling Delay> <Loops Number>
 
 *** Parameters:
 1) Tracking Process/es: all - for all processes,
                         app - for our processes,
                         aidc/web_idc/subagentx... - any specific process (use , for more then one processes)
 2) Polling Delay: number of seconds between each polling.
 3) Loops Number: How many intervals the script will run. value 0 means never stop (stop it by Ctrl+C).
 
 *** Examples:
 1) /etc/aidc/proc_track.sh aidc 10 100
    Track aidc process: run 100 intervals with 10 seconds delay between each interval.
 2) /etc/aidc/proc_track.sh app 3 0
    Infinite (Loops Number=0) track of ours processes for 3 intervals.	
 3) /etc/aidc/proc_track.sh all 5 10
    Track all process: run 10 intervals with 5 seconds delay between each interval.
 4) /etc/aidc/proc_track.sh aidc,web_idc 5 10
    Track both aidc,web_idc processes: run 10 intervals with 5 seconds delay between each interval.
	
 *** Analyzing the log:
 Output:
 The output is printed on the screen and on /etc/aidc/proc_track.log (It will replace the previos log file).
 The output contains these parameters:
 PID -     Process ID
 COMMAND - Process name.
 %MEM -    % of the memory this process consumes.
 RSS -     The non-swapped physical memory that a task has used (in kilo Bytes).
 VSZ -     Total Virtual Memory size (in kilo Bytes).
 SZ -	   Ignore it for now - approximate amount of swap space that would be required
           if the process were to dirty all writable pages and then be swapped out.
 %CPU -    % of CPU usage
 ELAPSED - Elapsed time since the process was started, in the form [[dd-]hh:]mm:ss.
 STAT -    For SW use (multi-character process state).
 
 If <Tracking Process> is 'all' or 'app' then more info is added to the output:
 Summary of free and used memory in the system (all processes) (use 50% of the memory is normal).
 Summary of the CPU usage. The there is no load in the system the %idle will be high (about 80%).

 1) How to find Memory Leaks (recommendations):
 Check the value of RSS, VSZ and %MEM 15 minutes after system is up (of a specific process). 
 compare it to the corresponding number after a night run. If it raised in more then 1% 
 then it might indicate a leakage -> continue to check and report to SW team.
 In general if the memory usage (RSS, VSS and %MEM) countinuesly keeps raising it indicate a leak.
 But if just toggled (for example process aidc consutmes 7.1% -> 7.3% -> 7.0% -> 7.2%) then its OK.

 2)CPU usage (analyzing recommendations):
 The average CPU usage in steady (normal) state of any process shouldn't be more than 10%.
 It is OK that the %CPU of snmpd process for example will be 80% when snmp is used.
 You should make sure that after finish the snmp test it comes back to its normal %CPU.
 In general, when the system is not in load check that no process consumes more than 10% for 
 a period of one minute. Check also that the %idle > 70%.

 here are the main processes and their main jobs:
 aidc - the main application
 --SNMP: 
 trapsenderd - send traps (v1 & v3).
 snmpd - handle snmp requests (standard net-snmp deamon).
 subagentx - our snmp agent, hadle private snmp requests.
 --WEB/NMS:
 web_idc - our web application
 nms_idc - Poly View agent
 lighttpd - standard web server
 --L2-PROTOCOLS:
 HSL; imi; nsm; onmd; mstpd - xSTP ,rmond - RMON.
 "
 exit 0
fi

# ********************* HELP END*****************************


echo "ps script start at: `date`"
echo "aidu version: `rpm -qa|grep aidu-idc`"
echo "IP: `ifconfig |grep Bcast`"


while true
do

 if [ "$1" = "all" ]; then
   echo "$cnt"
   free |grep -v Swap
   sar 1 1 |grep -1 idle
   ps -eo "$PARAMETERS"
 else 
   if [ "$1" = "app" ]; then
    echo "$cnt)"
    free |grep -v Swap
    sar 1 1 |grep -1 idle
    ps -C "$PROCESSES" -o "$PARAMETERS"
   else   
       if (( $cnt == 1 )); then	 
          ps -C "$1" -o "$PARAMETERS" |grep COMMAND	#take the headers only for the first inbterval
       fi
       ps -C "$1" -o "$PARAMETERS" --no-headers
   fi
 fi

 
 if (( $3 != 0 )); then	 
   if (( $cnt >= $3 )); then
     echo "finish on `date` after $cnt times"
     exit 0
   fi
 fi

 sleep "$2"
 ((cnt++))
done







