#!/usr/bin/perl -w
# Configuration Format Version: 1.0
# History log:
# 26.11.08	Ran Y.  -	initial version.	
# 31.05.09	Alex B. -	allow multiple files SFTP transfer.
# 09.03.10	Alex B. -	add File permissions and time copy options [for freeSSHd Windows server]..

use Net::SFTP::Foreign;
use File::Basename;
use Sys::Syslog qw( :DEFAULT setlogsock );

my $host = shift;
my $user = shift;
my $pass = shift;
my $src = shift;
my $dst = shift;
my $cmd = shift;
my $timeout = shift;
my %args = (
user => $user,
password => $pass,
timeout => $timeout
);

my %options = (
        copy_time => 1,
        copy_perm => 0
);

my $sftp = Net::SFTP::Foreign->new($host, %args);
if ($sftp->status != 0){
 print "sftp error: ".$sftp->error."\n";
 exit $sftp->status;
}

if ($cmd eq "put") {
	$sftp->put($src,$dst, %options) or die( "SFTP local file $src upload failed: ". $sftp->error .".\n" );
	log_msg( 'info', "file \'$src\' uploaded to \'$host\' SFTP server as \'$dst\'."  );
} elsif ($cmd eq "mput") {
	mput($sftp,$src);
} elsif ($cmd eq "get") {
	$sftp->get($src,$dst, %options) or die( "SFTP remote file $src download failed: " . $sftp->error .".\n" );
	log_msg( 'info', "file \'$src\' downloaded from \'$host\' SFTP server as \'$dst\'."  );
} elsif ($cmd eq "mget") {
	mget($sftp,$src);
} else {
	print STDERR "SFTP invocation error: unknown command $cmd\n";
	return 1;
}


if ($sftp->status != 0) {
	print "SFTP \'$cmd\' from \'$src\' to \'$dst\' error: ".$sftp->error."\n";
}
exit $sftp->status; 

sub mput {
	my($sftp,$pattern) = @_;
	if ($dst eq "/" or $dst eq "//") {
		$dst=".";
	} else {
		$sftp->setcwd($dst) or die( "SFTP remote change directory failed: " . $sftp->error .".\n" );
	}
	foreach my $file (glob($pattern)) {
		print basename($0).": uploading file \"$file\" ...\n";
		$sftp->put($file,basename($file), %options) or die( "SFTP file $file upload failed: ". $sftp->error .".\n" );
		log_msg( 'info', "file \'$file\' uploaded to \'$host\' SFTP server \'$dst\' directory."  );
	}
}

sub mget {
	my($sftp,$pattern) = @_;
	my $cwd = dirname($pattern);
	$sftp->setcwd($cwd) or die( "SFTP remote change directory failed: " . $sftp->error .".\n" );
	my @ls = $sftp->glob($pattern) or die( "file pattern matching failed: " . $sftp->error .".\n" );
	foreach my $item (@ls) {
		my $file = $item->{filename};
		print basename($0).": downloading remote file \"$file\" ...\n";
		$sftp->get($file, basename($file), %options) or die( "SFTP file $file download failed: " . $sftp->error .".\n" );
		log_msg( 'info', "file \'$file\' downloaded from \'$host\' SFTP server \'$cwd\' directory."  );
	}
}

sub log_msg {
        my ($priority, $msg) = @_; 
        return 0 unless ($priority =~ /info|err|debug/);

        setlogsock('unix');
	openlog(basename($0), '', 'user');
        syslog($priority, $msg);
        closelog();
        return 0;
}

